@@ -3,6 +3,7 @@
 import datetime
 import json
 import re
+import io
 import time
 
 import lxml.html
@@ -23,7 +24,7 @@ from understory import uri
 from understory import web
 
 __all__ = ["post", "get", "put", "delete",
-           "parse", "browser", "cache", "discover_link"]
+           "parse", "browser", "cache", "discover_link", "download"]
 
 displays = []
 browsers = []
@@ -233,7 +234,7 @@ class Transaction:
 
     @property
     def mf2json(self):
-        return Semantics(mf.parse(self.text, self.url))
+        return Semantics(mf.parse(str(self.text), self.url))
 
     @property
     def card(self):
@@ -393,6 +394,86 @@ def _make_element(html):
 
 
 class Browser:
+
+    def __init__(self, apps=None):
+        self.apps = apps
+
+    def get(self, url, data=None):
+        """GET resource at `url`."""
+        return self.handle("GET", url, data)
+
+    def put(self, url, data):
+        """PUT `data` to `url`."""
+        return self.handle("PUT", url, data)
+
+    def delete(self, url):
+        """DELETE resource at `url`."""
+        return self.handle("DELETE", url)
+
+    def post(self, url, data):
+        """POST `data` to `url`."""
+        return self.handle("POST", url, data)
+
+    def handle(self, method, url, data=None):
+        """
+        Handle a request to the outside web or a local WSGI app.
+
+        Use `requests` to reach the world wide web. Otherwise communicate
+        directly with WSGI apps in-process.
+
+        """
+        # TODO remove extraneous WSGI items
+        url = str(url)
+        if url.startswith(("http://", "https://")):
+            return getattr(requests, method.lower())(url)
+        response = []
+        hostname, _, path = url.partition("/")
+        path, _, query = path.partition("?")
+        app = self.apps[hostname]
+        if data:
+            wsgi_input = io.StringIO(json.dumps(data))
+        else:
+            wsgi_input = open("/dev/null")
+        env = {'HTTP_HOST': hostname,
+               'HTTP_CONNECTION': 'keep-alive',
+               'HTTP_ACCEPT': 'image/png,image/*;q=0.8,*/*;q=0.5',
+               'HTTP_USER_AGENT': 'Mozilla/5.0 (X11; Ubuntu; Linux i686; '
+                                  'rv:21.0) Gecko/20100101 Firefox/21.0',
+               'HTTP_COOKIE': 'session=99f81fd1f5e4c13b724043336f75'
+                              'e81ee51d806979220cc8cebe51bc9d0de292',
+               'HTTP_ACCEPT_LANGUAGE': 'en-US,en;q=0.5',
+               'HTTP_ACCEPT_ENCODING': 'gzip, deflate',
+               'PATH_INFO': f"/{path}",
+               'QUERY_STRING': query,
+               # XXX 'RAW_URI': f"/{path}",
+               'REMOTE_ADDR': '127.0.0.1',
+               # XXX 'REMOTE_PORT': '44612',
+               'REQUEST_METHOD': method,
+               # XXX 'SCRIPT_NAME': '',
+               'SERVER_PROTOCOL': 'HTTP/1.1',
+               'SERVER_SOFTWARE': 'gunicorn/0.17.4',
+               # XXX 'SERVER_NAME': 'localhost',
+               # XXX 'SERVER_PORT': '22739',
+               'wsgi.input': wsgi_input,
+               'wsgi.url_scheme': 'http'}
+
+        def start_response(status, headers):
+            response.append(status)
+            response.append(headers)
+
+        response.append(app(env, start_response))
+
+        transaction = Transaction(url, fetch=False)
+        transaction.status = web.tx.response.status
+        transaction.headers = web.tx.response.headers
+        transaction.text = web.tx.response.body
+        return transaction
+
+
+browser = Browser
+
+
+class Firefox:
     """Firefox via Selenium."""
 
     By = By
@@ -527,7 +608,7 @@ class Browser:
         #         f"<img class=screen src={self.shot_url()}></div>")
 
 
-browser = Browser
+firefox = Firefox
 
 
 # def shot(name, description):  # XXX , *browsers):
