@@ -448,16 +448,18 @@ class Form(dict):
             if isinstance(_data[key], list):
                 items = []
                 for item in _data[key]:
-                    if item.filename:
+                    if getattr(item, "filename", False):
                         items.append(File(item.filename, item))
                     else:
                         items.append(item.value)
                 self[key] = items
             else:
-                if _data[key].filename:
+                if getattr(_data[key], "filename", False):
                     value = File(key, _data[key])
-                else:
+                try:
                     value = _data.getfirst(key)
+                except AttributeError:
+                    value = _data[key]
                 if isinstance(defaults.get(key), list):
                     value = [value]
                 self[key] = value
@@ -745,39 +747,6 @@ class Application:
         self.path_args.update({k: v.replace(r"\!", nb60_re)
                                for k, v in path_args.items()})
 
-    def get(self, path):
-        """"""
-        env = {'HTTP_REFERER': 'http://localhost:22739/',
-               'HTTP_HOST': 'lahacker.net:22739',
-               'HTTP_CONNECTION': 'keep-alive',
-               'HTTP_ACCEPT': 'image/png,image/*;q=0.8,*/*;q=0.5',
-               'HTTP_USER_AGENT': 'Mozilla/5.0 (X11; Ubuntu; Linux i686; '
-                                  'rv:21.0) Gecko/20100101 Firefox/21.0',
-               'HTTP_COOKIE': 'session=99f81fd1f5e4c13b724043336f75'
-                              'e81ee51d806979220cc8cebe51bc9d0de292',
-               'HTTP_ACCEPT_LANGUAGE': 'en-US,en;q=0.5',
-               'HTTP_ACCEPT_ENCODING': 'gzip, deflate',
-               'PATH_INFO': path,
-               'QUERY_STRING': '',
-               'RAW_URI': '/',
-               'REMOTE_ADDR': '127.0.0.1',
-               'REMOTE_PORT': '44612',
-               'REQUEST_METHOD': 'GET',
-               'SCRIPT_NAME': '',
-               'SERVER_PROTOCOL': 'HTTP/1.1',
-               'SERVER_SOFTWARE': 'gunicorn/0.17.4',
-               'SERVER_NAME': 'localhost',
-               'SERVER_PORT': '22739',
-               'wsgi.input': open("/dev/null"),
-               'wsgi.url_scheme': 'http'}
-        response = []
-
-        def start_response(status, headers):
-            response.append(status)
-            response.append(headers)
-        response.append(self(env, start_response))
-        return tuple(response)
-
     def wrap(self, handler, when=None):
         """
         decorate a generator to run at various stages during the request
@@ -940,16 +909,16 @@ class Application:
         except Status as exc:
             tx.response.status = str(exc)  # TODO exc.status
             tx.response.body = exc.body
-            if exc.code == "201":
+            if exc.code == 201:
                 tx.response.headers.location = exc.location
-            if exc.code in ("301", "302", "303", "307", "308"):
+            elif exc.code in (301, 302, 303, 307, 308):
                 redirect_uri = apply_dns(str(tx.response.body))
                 if redirect_uri.startswith("/"):  # relative HTTP(S) URL
                     tx.response.headers.location = \
                         urllib.parse.quote(redirect_uri)
                 else:  # anything else (moz-extension, etc..)
                     tx.response.headers.location = redirect_uri
-            if exc.code == "405":
+            elif exc.code == 405:
                 tx.response.headers.allow = ", ".join(dict(exc.allowed))
         except Exception:
             # raise  # NOTE leave to bubble naturally -- debug in terminal
