@@ -11,10 +11,15 @@ from understory.web import tx
 
 from .. import webmention
 
+from .util import discover_post_type
+
 
 server = web.application("MicropubServer", mount_prefix="pub", db=False,
                          channel=r".+", entry=r".+", nickname=r"[A-Za-z0-9-]+",
                          filename=rf"{web.nb60_re}{{4}}.\w{{1,10}}")
+content = web.application("Content", mount_prefix=r"\d{{4}}", db=False,
+                          year=r"\d{4}", month=r"\d{2}", day=r"\d{2}",
+                          seconds=web.nb60_re + r"{,4}", slug=r"[\w_-]+")
 templates = web.templates(__name__)
 
 # TODO supported_types = {"RSVP": ["in-reply-to", "rsvp"]}
@@ -70,29 +75,6 @@ def route_unrouted(handler, app):
     yield
 
 
-def discover_post_type(properties):
-    """Return the discovered post type."""
-    if "photo" in properties:
-        post_type = "photo"
-    elif "bookmark-of" in properties:
-        post_type = "bookmark"
-    elif "like-of" in properties:
-        post_type = "like"
-    elif "follow-of" in properties:
-        post_type = "follow"
-    elif "identification-of" in properties:
-        post_type = "identification"
-    elif "rsvp" in properties:
-        post_type = "rsvp"
-    elif "in-reply-to" in properties:
-        post_type = "reply"
-    elif "name" in properties:
-        post_type = "article"
-    else:
-        post_type = "note"
-    return post_type
-
-
 def generate_vcard(nickname):
     """"""
     card = tx.pub.get_card(nickname)
@@ -172,10 +154,17 @@ class LocalClient:
     """A localized interface to the endpoint's backend."""
 
     def create(self, resource_type, **resource):
-        """Create a resource and return its permalink."""
+        """Create a resource."""
         for k, v in resource.items():
             if not isinstance(v, list):
                 resource[k] = [v]
+            flat_values = []
+            for v in resource[k]:
+                if isinstance(v, dict):
+                    v = dict(**v["properties"],
+                             type=v["type"][0].removeprefix("h-"))
+                flat_values.append(v)
+            resource[k] = flat_values
         if "published" in resource:
             resource["published"][0]["datetime"] = \
                 pendulum.from_format(resource["published"][0]["datetime"],
@@ -237,8 +226,8 @@ class LocalClient:
         resource.update(published=published, url=urls)
         resource["type"] = [resource_type]
         resource_kwargs = dict(resource=resource, version=web.nbrandom(10))
-        from pprint import pprint
-        pprint(resource_kwargs)
+        # XXX from pprint import pprint
+        # XXX pprint(resource_kwargs)
         tx.db.insert("resources", **resource_kwargs)
         web.publish("/recent", ".feed[-0:-0]", resource)
         for mention in mentions:
@@ -248,6 +237,31 @@ class LocalClient:
         #     web.enqueue(websub.publish)
         return urls[0]
 
+    def read(self, url):
+        """Return an entry with its metadata."""
+        if not url.startswith(("http://", "https://")):
+            url = f"/{url.strip('/')}"
+        try:
+            resource = tx.db.select("resources",
+                                    where="""json_extract(
+                                                 resources.resource,
+                                                 '$.url[0]') == ?""",
+                                    vals=[url])[0]
+        except IndexError:
+            try:
+                resource = tx.db.select("resources",
+                                        where="""json_extract(
+                                                     resources.resource,
+                                                     '$.alias[0]') == ?""",
+                                        vals=[url])[0]
+            except IndexError:
+                web.header("Content-Type", "text/html")  # TODO FIXME XXX
+                raise web.NotFound(templates.post_not_found(url))
+        r = resource["resource"]
+        if "entry" in r["type"]:
+            r["author"] = self.get_version(r["author"][0])["resource"]
+        return resource
+
     def update(self, url, add=None, replace=None, remove=None):
         """Update a resource."""
         now = web.utcnow()
@@ -276,31 +290,6 @@ class LocalClient:
                        '$.like-of[0].url') == ?"""
         return tx.db.select("resources", vals=[query, query], where=where)
 
-    def read(self, url):
-        """Return an entry with its metadata."""
-        if not url.startswith(("http://", "https://")):
-            url = f"/{url.rstrip('/')}"
-        try:
-            resource = tx.db.select("resources",
-                                    where="""json_extract(
-                                                 resources.resource,
-                                                 '$.url[0]') == ?""",
-                                    vals=[url])[0]
-        except IndexError:
-            try:
-                resource = tx.db.select("resources",
-                                        where="""json_extract(
-                                                     resources.resource,
-                                                     '$.alias[0]') == ?""",
-                                        vals=[url])[0]
-            except IndexError:
-                web.header("Content-Type", "text/html")  # TODO FIXME XXX
-                raise web.NotFound(templates.post_not_found(url))
-        r = resource["resource"]
-        if "entry" in r["type"]:
-            r["author"] = self.get_version(r["author"][0])["resource"]
-        return resource
-
     def get_version(self, version):
         """Return a snapshot of resource at given version."""
         return tx.db.select("resources", where="version = ?",
@@ -410,8 +399,8 @@ class MicropubEndpoint:
             response = {"q": ["category", "contact", "source", "syndicate-to"],
                         "media-endpoint": f"https://{tx.host.name}/pub/media",
                         "syndicate-to": syndication_endpoints,
-                        "visibility": ["public", "unlisted", "private"],
-                        "channels": generate_channels()}
+                        "visibility": ["public", "unlisted", "private"]}
+            # TODO "channels": generate_channels()}
         elif form.q == "channel":
             response = {"channels": generate_channels()}
         elif form.q == "source":
@@ -419,8 +408,10 @@ class MicropubEndpoint:
             if "search" in form:
                 response = {"items": [{"url": [r["resource"]["url"]]} for r in
                                       local_client.search(form.search)]}
-            if "url" in form:
+            elif "url" in form:
                 response = dict(local_client.read(form.url))
+            else:
+                pass  # TODO list all posts
         else:
             raise web.BadRequest("""unsupported query.
                                     check `q=config` for support.""")
@@ -949,3 +940,15 @@ class MediaFile:
         tx.db.delete("media", where="mid = ?", vals=[filepath.stem])
         filepath.unlink()
         return "deleted"
+
+
+@content.route(r"{month}/{day}/{seconds}(/{slug})?")
+class Permalink:
+    """An individual entry."""
+
+    def get(self):
+        resource = tx.pub.read(tx.request.uri.path)["resource"]
+        if resource["visibility"] == "private" and not tx.user.session:
+            raise web.Unauthorized(f"/auth?return_to={tx.request.uri.path}")
+        # mentions = web.indie.webmention.get_mentions(str(tx.request.uri))
+        return templates.entry(resource)  # , mentions)
