@@ -20,10 +20,12 @@ class Status(Exception):
 
     @property
     def code(self):
-        return self.__doc__.split()[0].strip(".")
+        return int(self.__doc__.split()[0].strip("."))
 
     @property
     def reason(self):
+        if self.code == 200:
+            return "OK"
         return re.sub("([A-Z])", r" \1", self.__class__.__name__).lstrip()
 
     def __str__(self):
